@@ -34,7 +34,7 @@ RETURNS = {
 }
 LOOKUPS = {
     "inorganicFertiliser": [
-        "mustIncludeId",
+        "complementaryTermIds",
         "nitrogenContent", "nitrogenContent-min", "nitrogenContent-max",
         "phosphateContentAsP2O5", "phosphateContentAsP2O5-min", "phosphateContentAsP2O5-max",
         "potassiumContentAsK2O", "potassiumContentAsK2O-min", "potassiumContentAsK2O-max"
@@ -75,7 +75,8 @@ def _input(term_id: str, value: float, min: float = None, max: float = None):
     return input
 
 
-def _include_term_ids(term_id: str): return non_empty_list((get_term_lookup(term_id, 'mustIncludeId') or '').split(';'))
+def _include_term_ids(term_id: str):
+    return non_empty_list((get_term_lookup(term_id, LOOKUPS['inorganicFertiliser'][0]) or '').split(';'))
 
 
 def _run_input(cycle: dict, input: dict):
@@ -138,12 +139,12 @@ def _should_run_input(cycle: dict, input: dict):
     ])
 
     for term_id in include_term_ids:
-        logRequirements(cycle, model=MODEL_LOG, term=term_id,
+        logRequirements(cycle, model=MODEL_LOG, term=term_id, model_key=MODEL_KEY,
                         nitrogenContent=nitrogenContent,
                         phosphateContentAsP2O5=phosphateContentAsP2O5,
                         potassiumContentAsK2O=potassiumContentAsK2O)
 
-        logShouldRun(cycle, MODEL_LOG, term_id, should_run)
+        logShouldRun(cycle, MODEL_LOG, term_id, should_run, model_key=MODEL_KEY)
     return should_run
 
 
