@@ -2,20 +2,21 @@
 Product Economic Value Share
 
 This model quantifies the relative economic value share of each marketable Product in a Cycle.
-Marketable Products are all Products in the Glossary with the exception of crop residue not sold.
+Marketable Products are all Products identified with the lookup "generateImpactAssessment = TRUE".
 
 It works in the following order:
-1. If revenue data are provided for all marketable products,
+1. For every product with no `value`, the `economicValueShare` is set to `0`.
+2. If revenue data are provided for all marketable products,
 the `economicValueShare` is directly calculated as the share of revenue of each Product;
-2. If the primary product is a crop and it is the only crop Product,
-`economicValueShare` is assigned based on a lookup table containing typical global average economic value shares
-drawn from [Poore & Nemecek (2018)](https://science.sciencemag.org/content/360/6392/987).
+3. Based on a lookup table containing typical global average economic value shares
+drawn from [Poore & Nemecek (2018)](https://science.sciencemag.org/content/360/6392/987), if all products have a
+corresponding value, the `economicValueShare` will be proportionally distributed among the products.
 """
-from hestia_earth.schema import TermTermType
 from hestia_earth.utils.model import find_term_match
 from hestia_earth.utils.tools import list_sum
 
 from hestia_earth.models.log import logRequirements, logShouldRun
+from hestia_earth.models.utils.blank_node import get_lookup_value
 from hestia_earth.models.utils.cycle import unique_currencies
 from .utils import lookup_share
 from .. import MODEL
@@ -25,9 +26,11 @@ REQUIREMENTS = {
         "products": [{
             "@type": "Product",
             "value": "",
-            "economicValueShare": "",
-            "revenue": "",
-            "currency": ""
+            "optional": {
+                "economicValueShare": "",
+                "revenue": "",
+                "currency": ""
+            }
         }],
         "optional": {
             "completeness.product": ""
@@ -42,7 +45,8 @@ RETURNS = {
 LOOKUPS = {
     "@doc": "Depending on the primary product [termType](https://hestia.earth/schema/Product#term)",
     "crop": "global_economic_value_share",
-    "excreta": "global_economic_value_share"
+    "excreta": "global_economic_value_share",
+    "animalProduct": "global_economic_value_share"
 }
 MODEL_KEY = 'economicValueShare'
 MAX_VALUE = 100.5
@@ -60,7 +64,7 @@ def _is_complete(cycle: dict): return cycle.get('completeness', {}).get('product
 def _no_yield(product): return list_sum(product.get('value', [-1]), -1) == 0
 
 
-def _total_revenue(products: list): return sum([p.get(MODEL_KEY, 0) for p in products])
+def _total_evs(products: list): return sum([p.get(MODEL_KEY, 0) for p in products])
 
 
 def _product_with_value(product: dict):
@@ -72,18 +76,33 @@ def _rescale_value(products: list, total: float):
     return list(map(lambda p: {**p, MODEL_KEY: p.get(MODEL_KEY) * 100 / total}, products))
 
 
+def _should_run_by_default(cycle: dict, products: list):
+    run_by = 'default'
+    # only run if all products have the lookup data or value will be incorrect
+    products = list(map(_product_with_value, products))
+    all_with_economicValueShare = all([p.get(MODEL_KEY) is not None for p in products])
+
+    should_run = all([all_with_economicValueShare])
+
+    for p in products:
+        term_id = p.get('term', {}).get('@id')
+        logRequirements(cycle, model=MODEL, term=term_id, key=MODEL_KEY, by=run_by,
+                        all_with_economicValueShare=all_with_economicValueShare)
+        logShouldRun(cycle, MODEL, term_id, should_run, key=MODEL_KEY, by=run_by)
+
+    return should_run
+
+
 def _run_by_default(cycle: dict, products: list):
     run_by = 'default'
     is_complete = _is_complete(cycle)
     products = list(map(_product_with_value, products))
-    # remove results where lookup share was not found
-    results = list(filter(lambda p: p.get(MODEL_KEY) is not None, products))
     # only return list if the new total of evs is not above threshold
-    total_revenue = _total_revenue(results)
-    below_max_threshold = total_revenue <= MAX_VALUE
-    should_rescale = is_complete and MIN_COMPLETE_VALUE <= total_revenue <= MAX_VALUE
-    above_min_threshold = True if should_rescale else total_revenue >= MIN_VALUE if is_complete else True
-    results = _rescale_value(results, total_revenue) if should_rescale else results
+    total_economicValueShare = _total_evs(products)
+    below_max_threshold = total_economicValueShare <= MAX_VALUE
+    should_rescale = is_complete and MIN_COMPLETE_VALUE <= total_economicValueShare <= MAX_VALUE
+    above_min_threshold = True if should_rescale else total_economicValueShare >= MIN_VALUE if is_complete else True
+    results = _rescale_value(products, total_economicValueShare) if should_rescale else products
 
     should_run = all([below_max_threshold, above_min_threshold])
 
@@ -93,7 +112,7 @@ def _run_by_default(cycle: dict, products: list):
         logRequirements(cycle, model=MODEL, term=term_id, key=MODEL_KEY, by=run_by,
                         below_max_threshold=below_max_threshold,
                         above_min_threshold=above_min_threshold,
-                        total_revenue=total_revenue)
+                        total_economicValueShare=total_economicValueShare)
         logShouldRun(cycle, MODEL, term_id, p_should_run, key=MODEL_KEY, by=run_by)
 
     return results if should_run else []
@@ -109,8 +128,8 @@ def _run_by_revenue(products: list):
 def _should_run_by_revenue(cycle: dict, products: list):
     run_by = 'revenue'
     is_complete = _is_complete(cycle)
-    total_value = _total_revenue(products)
-    below_threshold = total_value < MAX_VALUE
+    total_economicValueShare = _total_evs(products)
+    below_threshold = total_economicValueShare < MAX_VALUE
     # ignore products with no yield
     products = list(filter(lambda p: not _no_yield(p), products))
     currencies = unique_currencies(cycle)
@@ -122,7 +141,7 @@ def _should_run_by_revenue(cycle: dict, products: list):
         term_id = p.get('term', {}).get('@id')
         logRequirements(cycle, model=MODEL, term=term_id, key=MODEL_KEY, by=run_by,
                         is_complete=is_complete,
-                        total_value=total_value,
+                        total_economicValueShare=total_economicValueShare,
                         below_threshold=below_threshold,
                         all_with_revenue=all_with_revenue,
                         currencies=';'.join(currencies),
@@ -133,14 +152,14 @@ def _should_run_by_revenue(cycle: dict, products: list):
 
 
 def _run_single_missing_evs(products: list):
-    total_value = _total_revenue(products)
+    total_value = _total_evs(products)
     return list(map(lambda p: _product(p, 100 - total_value) if p.get(MODEL_KEY) is None else p, products))
 
 
 def _should_run_single_missing_evs(cycle: dict, products: list):
     run_by = '1-missing-evs'
     is_complete = _is_complete(cycle)
-    total_value = _total_revenue(products)
+    total_value = _total_evs(products)
     # ignore products with no yield
     products = list(filter(lambda p: not _no_yield(p), products))
     missing_values = [p for p in products if p.get(MODEL_KEY) is None]
@@ -175,20 +194,17 @@ def _should_run_no_value(cycle: dict, product: dict):
 
 
 def _should_have_evs(product: dict):
-    term_type = product.get('term', {}).get('termType')
-    return term_type not in [
-        TermTermType.CROPRESIDUE.value,
-        TermTermType.EXCRETA.value
-    ]
+    should_generate_ia = get_lookup_value(product.get('term', {}), 'generateImpactAssessment')
+    return str(should_generate_ia).lower() != 'false'
 
 
 def run(cycle: dict):
     products = cycle.get('products', [])
-    # skip any product that will never has value
+    # skip any product that is not marketable
     products = list(filter(_should_have_evs, products))
     products = list(map(lambda p: _product(p, 0) if _should_run_no_value(cycle, p) else p, products))
     return (
         _run_single_missing_evs(products) if _should_run_single_missing_evs(cycle, products) else
         _run_by_revenue(products) if _should_run_by_revenue(cycle, products) else
-        _run_by_default(cycle, products)
+        _run_by_default(cycle, products) if _should_run_by_default(cycle, products) else []
     )
