@@ -11,7 +11,8 @@ REQUIREMENTS = {
     "Site": {
         "or": [
             {"latitude": "", "longitude": ""},
-            {"boundary": {}}
+            {"boundary": {}},
+            {"region": {"@type": "Term", "termType": "region"}}
         ]
     }
 }
@@ -22,20 +23,20 @@ MODEL_KEY = 'aware'
 EE_PARAMS = {
     'collection': 'AWARE',
     'ee_type': 'vector',
-    'field': 'Name'
+    'fields': 'Name'
 }
 
 
 def _download(site: dict):
-    return download(MODEL_KEY, site, EE_PARAMS, by_region=False).get(EE_PARAMS['field'])
+    return download(MODEL_KEY, site, EE_PARAMS)
 
 
 def _run(site: dict): return _download(site)
 
 
 def _should_run(site: dict):
-    contains_geospatial_data = has_geospatial_data(site, by_region=False)
-    below_max_area_size = should_download(MODEL_KEY, site, by_region=False)
+    contains_geospatial_data = has_geospatial_data(site)
+    below_max_area_size = should_download(MODEL_KEY, site)
 
     logRequirements(site, model=MODEL, model_key=MODEL_KEY,
                     contains_geospatial_data=contains_geospatial_data,
