@@ -51,7 +51,7 @@ def _get_input_value_from_term(inputs: list, term_id: str):
 def _get_value(cycle: dict, term: dict, irrigation_ids: list):
     term_id = term.get('@id', '')
 
-    data_complete = _is_term_type_complete(cycle, {'termType': TermTermType.WATER.value})
+    data_complete = _is_term_type_complete(cycle, TermTermType.WATER)
 
     if data_complete:
         value = sum([_get_input_value_from_term(cycle.get('inputs', []), term_id) for term_id in irrigation_ids])
