@@ -14,7 +14,7 @@ from . import MODEL
 REQUIREMENTS = {
     "Cycle": {
         "completeness.animalFeed": "True",
-        "completeness.grazedForage": "True",
+        "completeness.freshForage": "True",
         "or": [
             {
                 "animals": [{
@@ -54,13 +54,19 @@ REQUIREMENTS = {
 LOOKUPS = {
     "animalProduct": [
         "digestibility",
-        "percentageYmMethaneConversionFactorEntericFermentationIPCC2019",
-        "percentageYmMethaneConversionFactorEntericFermentationIPCC2019-sd"
+        "percentageYmMethaneConversionFactorEntericFermentation",
+        "percentageYmMethaneConversionFactorEntericFermentation-sd",
+        "defaultPercentageYmMethaneConversionFactorEntericFermentation",
+        "defaultPercentageYmMethaneConversionFactorEntericFermentation-min",
+        "defaultPercentageYmMethaneConversionFactorEntericFermentation-max"
     ],
     "liveAnimal": [
         "digestibility",
-        "percentageYmMethaneConversionFactorEntericFermentationIPCC2019",
-        "percentageYmMethaneConversionFactorEntericFermentationIPCC2019-sd"
+        "percentageYmMethaneConversionFactorEntericFermentation",
+        "percentageYmMethaneConversionFactorEntericFermentation-sd",
+        "defaultPercentageYmMethaneConversionFactorEntericFermentation",
+        "defaultPercentageYmMethaneConversionFactorEntericFermentation-min",
+        "defaultPercentageYmMethaneConversionFactorEntericFermentation-max"
     ],
     "crop-property": ["neutralDetergentFibreContent", "energyContentHigherHeatingValue"]
 }
@@ -75,36 +81,36 @@ RETURNS = {
 TERM_ID = 'ch4ToAirEntericFermentation'
 TIER = EmissionMethodTier.TIER_2.value
 METHANE_EC = 55.65  # MJ/kg CH4
-DEFAULT_YM = {
-    'value': 6.125,
-    'min': 5.7,
-    'max': 6.5,
-    'description': 'Average Ym factor of 6.125% used as data missing to differentiate Ym.'
-}
 
 
-def _emission(value: float, sd: float = None, min: float = None, max: float = None, default: bool = False):
+def _emission(value: float, sd: float = None, min: float = None, max: float = None, description: str = None):
     emission = _new_emission(TERM_ID, MODEL)
     emission['value'] = [value]
     if sd is not None:
         emission['sd'] = [sd]
         emission['statsDefinition'] = EmissionStatsDefinition.MODELLED.value
-    if default:
+    if min is not None:
         emission['min'] = [min]
-        emission['max'] = [max]
-        emission['description'] = DEFAULT_YM.get('description')
         emission['statsDefinition'] = EmissionStatsDefinition.MODELLED.value
+    if max is not None:
+        emission['max'] = [max]
+        emission['statsDefinition'] = EmissionStatsDefinition.MODELLED.value
+    if description:
+        emission['description'] = description
     emission['methodTier'] = TIER
     return emission
 
 
-def _run(feed: float, enteric_factor: float = None, enteric_sd: float = None):
-    value = (feed * ((enteric_factor or DEFAULT_YM.get('value')) / 100)) / METHANE_EC
-    min = (feed * (DEFAULT_YM.get('min') / 100)) / METHANE_EC
-    max = (feed * (DEFAULT_YM.get('max') / 100)) / METHANE_EC
+def _run(feed: float, enteric_factor: float = None, enteric_sd: float = None, default_values=(None, None, None)):
+    value = (feed * ((enteric_factor or default_values[0]) / 100)) / METHANE_EC
+    min = (feed * (default_values[1] / 100)) / METHANE_EC if all([enteric_factor is None, default_values[1]]) else None
+    max = (feed * (default_values[2] / 100)) / METHANE_EC if all([enteric_factor is None, default_values[2]]) else None
+    description = f"Average Ym factor of {default_values[0]}% used as data missing to differentiate Ym." if all([
+        enteric_factor is None,
+        default_values[0] is not None
+    ]) else None
     return [
-        _emission(value, sd=enteric_sd, default=False) if enteric_factor
-        else _emission(value, min=min, max=max, default=True)
+        _emission(value, enteric_sd, min, max, description)
     ]
 
 
@@ -112,7 +118,7 @@ DE_NDF_MAPPING = {
     'high_DE_low_NDF': lambda DE, NDF: DE >= 70 and NDF < 35,
     'high_DE_high_NDF': lambda DE, NDF: DE >= 70 and NDF >= 35,
     'medium_DE_high_NDF': lambda DE, NDF: 63 <= DE < 70 and NDF > 37,
-    'low_DE_high_NDF': lambda DE, NDF: DE < 62 and NDF >= 38
+    'low_DE_high_NDF': lambda DE, NDF: DE < 63 and NDF > 38
 }
 MILK_YIELD_MAPPING = {
     'high_DE_low_NDF': lambda milk_yield: milk_yield > 8500,
@@ -122,11 +128,11 @@ MILK_YIELD_MAPPING = {
 }
 
 DE_MAPPING = {
-    'high_medium_DE': lambda DE, _: DE > 63,
+    'high_medium_DE': lambda DE, _: DE > 62,
     'medium_DE': lambda DE, _: DE > 62 and DE < 72,
     'low_DE': lambda DE, _: DE <= 62,
     'high_DE': lambda DE, _: DE >= 72,
-    'high_DE_ionophore': lambda DE, ionophore: DE > 75 and ionophore
+    'high_DE_ionophore': lambda DE, ionophore: DE >= 75 and ionophore
 }
 
 
@@ -155,7 +161,7 @@ def _extract_groupped_data(value: str, DE: float, NDF: float, ionophore: bool, m
     debugValues({}, model=MODEL, term=TERM_ID,
                 value_key=value_key)
 
-    return safe_parse_float(extract_grouped_data(value, value_key))
+    return safe_parse_float(extract_grouped_data(value, value_key), None)
 
 
 def _get_lookup_value(lookup, term: dict, lookup_col: str, DE: float, NDF: float, ionophore: bool, milk_yield: float):
@@ -191,9 +197,17 @@ def _is_ionophore(cycle: dict, total_feed: float):
     return has_input and maize_feed_ratio >= 0.9
 
 
+def _get_default_values(lookup, term: dict):
+    term_id = term.get('@id')
+    value = get_table_value(lookup, 'termid', term_id, column_name(LOOKUPS['liveAnimal'][3])) if term_id else None
+    min = get_table_value(lookup, 'termid', term_id, column_name(LOOKUPS['liveAnimal'][4])) if term_id else None
+    max = get_table_value(lookup, 'termid', term_id, column_name(LOOKUPS['liveAnimal'][5])) if term_id else None
+    return safe_parse_float(value, None), safe_parse_float(min, None), safe_parse_float(max, None)
+
+
 def _should_run(cycle: dict):
     is_animalFeed_complete = cycle.get('completeness', {}).get('animalFeed', False) is True
-    is_grazedForage_complete = cycle.get('completeness', {}).get('grazedForage', False) is True
+    is_freshForage_complete = cycle.get('completeness', {}).get('freshForage', False) is True
 
     primary_product = find_primary_product(cycle) or {}
     term = primary_product.get('term', {})
@@ -213,8 +227,7 @@ def _should_run(cycle: dict):
     # only keep inputs that have a positive value
     inputs = list(filter(lambda i: list_sum(i.get('value', [])) > 0, feed_inputs))
     DE = (
-        get_total_value_converted_with_min_ratio(MODEL, TERM_ID, cycle, inputs, DE_type) if isinstance(DE_type, str)
-        else None
+        get_total_value_converted_with_min_ratio(MODEL, TERM_ID, cycle, inputs, DE_type) if DE_type else None
     ) or get_default_digestibility(MODEL, TERM_ID, cycle)
     NDF = get_total_value_converted_with_min_ratio(MODEL, TERM_ID, cycle, inputs, 'neutralDetergentFibreContent')
 
@@ -225,6 +238,8 @@ def _should_run(cycle: dict):
         lookup, term, LOOKUPS['liveAnimal'][2], DE, NDF, ionophore, milk_yield
     ), None)
 
+    default_values = _get_default_values(lookup, term)
+
     debugValues(cycle, model=MODEL, term=TERM_ID,
                 DE_type=DE_type,
                 digestibility=DE,
@@ -232,18 +247,22 @@ def _should_run(cycle: dict):
                 ionophore=ionophore,
                 milk_yield=milk_yield,
                 enteric_factor=enteric_factor,
-                enteric_sd=enteric_sd)
+                enteric_sd=enteric_sd,
+                default_values=default_values)
 
     logRequirements(cycle, model=MODEL, term=TERM_ID,
                     term_type_animalFeed_complete=is_animalFeed_complete,
-                    term_type_grazedForage_complete=is_grazedForage_complete,
+                    term_type_freshForage_complete=is_freshForage_complete,
                     total_feed_in_MJ=total_feed)
 
-    should_run = all([is_animalFeed_complete, is_grazedForage_complete, total_feed])
+    should_run = all([
+        is_animalFeed_complete, is_freshForage_complete, total_feed,
+        enteric_factor or default_values[0]
+    ])
     logShouldRun(cycle, MODEL, TERM_ID, should_run, methodTier=TIER)
-    return should_run, total_feed, enteric_factor, enteric_sd
+    return should_run, total_feed, enteric_factor, enteric_sd, default_values
 
 
 def run(cycle: dict):
-    should_run, feed, enteric_factor, enteric_sd = _should_run(cycle)
-    return _run(feed, enteric_factor, enteric_sd) if should_run else []
+    should_run, feed, enteric_factor, enteric_sd, default_values = _should_run(cycle)
+    return _run(feed, enteric_factor, enteric_sd, default_values) if should_run else []
