@@ -7,6 +7,7 @@ from hestia_earth.models.log import debugValues, logRequirements, logShouldRun
 from hestia_earth.models.utils.term import get_lookup_value
 from hestia_earth.models.utils.emission import _new_emission
 from hestia_earth.models.utils.product import has_flooded_rice
+from hestia_earth.models.utils.organicFertiliser import get_cycle_inputs as get_organicFertiliser_inputs
 from . import MODEL
 
 REQUIREMENTS = {
@@ -17,7 +18,19 @@ REQUIREMENTS = {
             "country": {"@type": "Term", "termType": "region"}
         },
         "optional": {
-            "inputs": [{"@type": "Input", "value": "", "term.termType": "organicFertiliser"}],
+            "inputs": [
+                {
+                    "@type": "Input",
+                    "value": "",
+                    "term.termType": "organicFertiliser"
+                },
+                {
+                    "@type": "Input",
+                    "value": "",
+                    "term.termType": "fertiliserBrandName",
+                    "properties": [{"@type": "Property", "value": "", "key.termType": "organicFertiliser"}]
+                }
+            ],
             "products": [{"@type": "Product", "value": "", "term.@id": "aboveGroundCropResidueIncorporated"}],
             "practices": [
                 {"@type": "Practice", "value": "", "term.termType": "cropResidueManagement"},
@@ -99,7 +112,7 @@ def _get_fertiliser_value(input: dict, suffix: str = ''):
 
 def _calculate_SFo(cycle: dict, suffix: str = ''):
     cropResidue = _get_cropResidue_value(cycle, suffix)
-    fertilisers = filter_list_term_type(cycle.get('inputs', []), TermTermType.ORGANICFERTILISER)
+    fertilisers = get_organicFertiliser_inputs(cycle)
     fert_value = list_sum([_get_fertiliser_value(i, suffix) for i in fertilisers])
     return (1 + (fert_value/1000) + (cropResidue/1000)) ** 0.59
 
