@@ -11,13 +11,8 @@ from . import MODEL
 
 REQUIREMENTS = {
     "Cycle": {
-        "or": {
-            "completeness.fertiliser": "",
-            "inputs": [{"@type": "Input", "value": "", "term.termType": "inorganicFertiliser"}]
-        },
-        "optional": {
-            "inputs": [{"@type": "Input", "value": "", "term.@id": "inorganicNitrogenFertiliserUnspecifiedKgN"}]
-        }
+        "completeness.fertiliser": "",
+        "inputs": [{"@type": "Input", "value": "", "term.termType": "inorganicFertiliser"}]
     }
 }
 RETURNS = {
@@ -61,7 +56,7 @@ def _run(cycle: dict, urea_values: list):
 def _get_urea_values(cycle: dict, inputs: list, term_id: str):
     inputs = list(filter(lambda i: i.get('term', {}).get('@id') == term_id, inputs))
     values = [list_sum(i.get('value'), 0) for i in inputs if len(i.get('value', [])) > 0]
-    return [0] if len(inputs) == 0 and _is_term_type_complete(cycle, {'termType': 'fertiliser'}) else values
+    return [0] if len(inputs) == 0 and _is_term_type_complete(cycle, 'fertiliser') else values
 
 
 def _should_run(cycle: dict):
@@ -76,7 +71,11 @@ def _should_run(cycle: dict):
     urea_values = [{'id': id, 'values': _get_urea_values(cycle, inputs, id)} for id in term_ids] + ([
         {'id': 'ureaKgN', 'values': [urea_unspecified_as_n * urea_share]},
         {'id': 'ureaAmmoniumNitrateKgN', 'values': [urea_unspecified_as_n * uan_share]}
-    ] if urea_unspecified_as_n > 0 else [])
+    ] if all([
+        urea_share,
+        uan_share,
+        urea_unspecified_as_n > 0
+    ]) else [])
     has_urea_value = any([len(data.get('values')) > 0 for data in urea_values])
 
     logRequirements(cycle, model=MODEL, term=TERM_ID,
