@@ -80,11 +80,11 @@ def _should_run(cycle: dict):
     logRequirements(cycle, model=MODEL, term=TERM_ID,
                     country=country,
                     has_flooded_rice=flooded_rice,
-                    cycleDuration=cycleDuration,
+                    cycleDuration=cycle.get('cycleDuration'),
                     croppingDuration=croppingDuration,
                     croppingDuration_below_cycleDuration=croppingDuration_below_cycleDuration)
 
-    should_run = all([country, cycleDuration, flooded_rice])
+    should_run = all([country, cycleDuration > 0, croppingDuration_below_cycleDuration, flooded_rice])
     logShouldRun(cycle, MODEL, TERM_ID, should_run)
     return should_run, country
 
