@@ -1,12 +1,9 @@
-import os
 from inspect import getmembers, isfunction
 import json
+from hestia_earth.utils.tools import flatten
 
-from hestia_earth.models.log import logger
 from hestia_earth.models.utils import term
 
-CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
-RESULTS_PATH = os.path.join(CURRENT_DIR, 'search-results.json')
 IGNORE_FUNC = ['get_lookup_value', 'get_table_value']
 
 
@@ -14,7 +11,7 @@ def _map_results(results):
     # returning the whole term
     return [results] if isinstance(results, dict) else (
         {'@type': 'Term', '@id': results} if isinstance(results, str) else
-        list(map(_map_results, results)) if isinstance(results, list) else
+        flatten(map(_map_results, results)) if isinstance(results, list) else
         None
     )
 
@@ -48,24 +45,33 @@ def create_search_results():
     return list(map(_create_search_result, funcs))
 
 
-def _load_results():
-    with open(RESULTS_PATH) as f:
+def _load_results(filepath: str):
+    with open(filepath) as f:
         return json.load(f)
 
 
-def _find_search_result(query: dict):
-    search_results = _load_results()
-    res = next((n for n in search_results if n['query'] == query), {})
-    logger.debug('mocking search result: %s', res)
-    return res.get('results', [])
+def _find_search_result(filepath: str, query: dict):
+    search_results = _load_results(filepath)
+    res = next((n for n in search_results if n['query'] == query), None)
+    return None if res is None else res.get('results', [])
 
 
-def _fake_search(query: dict, *_a, **_b): return _find_search_result(query)
+def _mocked_search(original_func, filepath: str):
+    def mock(query: dict, **kwargs):
+        result = _find_search_result(filepath, query)
+        return original_func(query, **kwargs) if result is None else result
+    return mock
 
 
-def _fake_find_node(_n, query: dict, *_a, **_b): return _find_search_result(query)
+def _mocked_find_node(original_func, filepath: str):
+    def mock(node_type: str, query: dict, **kwargs):
+        result = _find_search_result(filepath, query)
+        return original_func(node_type, query, **kwargs) if result is None else result
+    return mock
 
 
-def mock():
-    term.search = _fake_search
-    term.find_node = _fake_find_node
+def mock(filepath: str):
+    original_search = term.search
+    term.search = _mocked_search(original_search, filepath)
+    original_find_node = term.find_node
+    term.find_node = _mocked_find_node(original_find_node, filepath)
