@@ -23,7 +23,7 @@ TERM_ID = 'freshWater'
 
 def _measurement():
     data = _new_measurement(TERM_ID)
-    data['value'] = [100]
+    data['value'] = [True]
     data['methodClassification'] = MeasurementMethodClassification.MODELLED_USING_OTHER_MEASUREMENTS.value
     return data
 
