@@ -1,6 +1,8 @@
 from hestia_earth.schema import EmissionMethodTier
 
+from hestia_earth.models.log import logRequirements, logShouldRun
 from hestia_earth.models.utils.emission import _new_emission
+from hestia_earth.models.utils.completeness import _is_term_type_complete
 from hestia_earth.models.utils.cycle import get_organic_fertiliser_N_total
 from .n2OToAirSoilFlux import _get_value, _should_run
 from . import MODEL
@@ -8,15 +10,23 @@ from . import MODEL
 REQUIREMENTS = {
     "Cycle": {
         "completeness.fertiliser": "True",
-        "inputs": [{
-            "@type": "Input",
-            "value": "",
-            "term.units": ["kg", "kg N"],
-            "term.termType": "organicFertiliser",
-            "optional": {
-                "properties": [{"@type": "Property", "value": "", "term.@id": "nitrogenContent"}]
+        "inputs": [
+            {
+                "@type": "Input",
+                "value": "",
+                "term.units": ["kg", "kg N"],
+                "term.termType": "organicFertiliser",
+                "optional": {
+                    "properties": [{"@type": "Property", "value": "", "term.@id": "nitrogenContent"}]
+                }
+            },
+            {
+                "@type": "Input",
+                "value": "",
+                "term.termType": "fertiliserBrandName",
+                "properties": [{"@type": "Property", "value": "", "key.termType": "organicFertiliser"}]
             }
-        }],
+        ],
         "site": {
             "@type": "Site",
             "measurements": [
@@ -61,5 +71,13 @@ def _run(cycle: dict, content_list_of_items: list, N_total: float):
 
 
 def run(cycle: dict):
-    should_run, N_total, content_list_of_items = _should_run(cycle, TERM_ID, TIER)
+    default_should_run, N_total, content_list_of_items = _should_run(cycle, TERM_ID, TIER)
+    term_type_complete = _is_term_type_complete(cycle, 'fertiliser')
+
+    logRequirements(cycle, model=MODEL, term=TERM_ID,
+                    term_type_fertiliser_complete=term_type_complete)
+
+    should_run = all([default_should_run, term_type_complete])
+    logShouldRun(cycle, MODEL, TERM_ID, should_run)
+
     return _run(cycle, content_list_of_items, N_total) if should_run else []
