@@ -1,4 +1,6 @@
 from os.path import dirname, abspath
+from collections.abc import Generator, Iterable
+from itertools import tee
 import sys
 import datetime
 from functools import reduce
@@ -7,6 +9,7 @@ from typing import Any, Union
 from hestia_earth.schema import SchemaType
 from hestia_earth.utils.api import download_hestia
 from hestia_earth.utils.model import linked_node
+from hestia_earth.utils.tools import flatten, non_empty_list
 
 from .constant import Units
 
@@ -31,14 +34,13 @@ def _include_model(node: dict, term_id: str):
     return {**node, **({} if term.get('@id') is None else {'model': linked_node(term)})}
 
 
-def _include_method(node: dict, term_id: Union[None, str, dict]):
+def _include_method(node: dict, term_id: Union[None, str, dict], key='method'):
     term = (download_hestia(term_id) or {}) if isinstance(term_id, str) else term_id
-    return node | ({} if term is None or term.get('@id') is None else {'method': linked_node(term)})
+    return node | ({} if term is None or term.get('@id') is None else {key: linked_node(term)})
 
 
 def _include_methodModel(node: dict, term_id: str):
-    term = download_hestia(term_id) or {}
-    return {**node, **({} if term.get('@id') is None else {'methodModel': linked_node(term)})}
+    return _include_method(node, term_id=term_id, key='methodModel')
 
 
 def _run_in_serie(data: dict, models: list): return reduce(lambda prev, model: model(prev), models, data)
@@ -59,7 +61,7 @@ def _filter_list_term_unit(values: list, unit: Any):
 
 def is_from_model(node: dict) -> bool:
     """
-    Check if the Blank Node came from one of the Hestia Models.
+    Check if the Blank Node came from one of the HESTIA Models.
 
     Parameters
     ----------
@@ -120,15 +122,6 @@ def get_kg_term_units(term_id: str, units: str):
     }.get(units, lambda x: None)(term_id)
 
 
-def _get_by_key(x, y):
-    return x if x is None else (
-        x.get(y) if isinstance(x, dict) else list(map(lambda v: get_dict_key(v, y), x))
-    )
-
-
-def get_dict_key(value: dict, key: str): return reduce(lambda x, y: _get_by_key(x, y), key.split('.'), value)
-
-
 def first_day_of_month(year: int, month: int):
     return datetime.date(int(year), int(month), 1)
 
@@ -138,3 +131,28 @@ def last_day_of_month(year: int, month: int):
     return datetime.date(int(year), 12, 31) if month == 12 else (
         datetime.date(int(year) + int(int(month) / 12), (int(month) % 12) + 1, 1) - datetime.timedelta(days=1)
     )
+
+
+def flatten_args(args) -> list:
+    """
+    Flatten the input args into a single list.
+    """
+    return non_empty_list(flatten([list(arg) if is_iterable(arg) else [arg] for arg in args]))
+
+
+def is_iterable(arg) -> bool:
+    """
+    Return `True` if the input arg is an instance of an `Iterable` (excluding `str` and `bytes`) or a `Generator`, else
+    return `False`.
+    """
+    return isinstance(arg, (Iterable, Generator)) and not isinstance(arg, (str, bytes))
+
+
+def pairwise(iterable):
+    """
+    from https://docs.python.org/3.9/library/itertools.html#itertools-recipes
+    s -> (s0,s1), (s1,s2), (s2, s3), ...
+    """
+    a, b = tee(iterable)
+    next(b, None)
+    return zip(a, b)
