@@ -1,11 +1,13 @@
+import calendar
 from calendar import monthrange
 from collections import defaultdict
 from collections.abc import Iterable
-from datetime import datetime
+from datetime import datetime, timedelta
+from uuid import uuid4
+
 from dateutil.relativedelta import relativedelta
 from enum import Enum
 from functools import reduce
-from statistics import mode, mean
 from typing import (
     Any,
     List,
@@ -14,7 +16,9 @@ from typing import (
     Optional,
     Union
 )
+from hestia_earth.schema import TermTermType
 from hestia_earth.utils.api import download_hestia
+from hestia_earth.utils.model import filter_list_term_type
 from hestia_earth.utils.tools import (
     flatten,
     list_sum,
@@ -22,9 +26,10 @@ from hestia_earth.utils.tools import (
     safe_parse_float,
     non_empty_list
 )
+from hestia_earth.utils.blank_node import ArrayTreatment, get_node_value
 
 from ..log import debugValues, log_as_table
-from . import is_from_model, _filter_list_term_unit
+from . import is_from_model, _filter_list_term_unit, is_iterable, _omit
 from .constant import Units
 from .property import get_node_property, get_node_property_value
 from .lookup import (
@@ -36,6 +41,63 @@ from .lookup import (
 from .term import get_lookup_value
 
 
+def merge_blank_nodes(source: list, new_values: list):
+    """
+    Merge a list of blank nodes into an existing list of blank nodes.
+    Warning: we only consider the `term.@id` here, and not the full list of properties that make the nodes unique.
+    This should only be used when merging simple list of nested blank nodes.
+    """
+    for new_value in non_empty_list(new_values):
+        term_id = new_value.get('term', {}).get('@id')
+        index = next(
+            (i for i, data in enumerate(source) if data.get('term', {}).get('@id') == term_id),
+            None
+        )
+        if index is None:
+            source.append(new_value)
+        else:
+            source[index] = source[index] | new_value
+    return source
+
+
+def lookups_logs(model: str, blank_nodes: list, lookups_per_termType: dict, **log_args):
+    def mapper(blank_node: dict):
+        term = blank_node.get('term', {})
+        term_id = term.get('@id')
+        term_type = term.get('termType')
+        lookups = lookups_per_termType.get(term_type, [])
+        lookups = lookups if isinstance(lookups, list) else [lookups]
+
+        def _reduce_lookups_logs(logs: dict, column: str):
+            lookup_value = get_lookup_value(term, column, model=model, **log_args)
+            return logs | {column: lookup_value}
+
+        return reduce(_reduce_lookups_logs, lookups, {'id': term_id})
+
+    logs = list(map(mapper, blank_nodes))
+
+    return log_as_table(logs)
+
+
+def properties_logs(blank_nodes: list, properties: Union[dict, list]):
+    def mapper(blank_node: dict):
+        term = blank_node.get('term', {})
+        term_id = term.get('@id')
+        term_type = term.get('termType')
+        props = properties.get(term_type, []) if isinstance(properties, dict) else properties
+        props = props if isinstance(props, list) else [props]
+
+        def _reduce_properties_logs(logs: dict, prop: str):
+            value = get_node_property(blank_node, prop).get('value')
+            return logs | {prop: value}
+
+        return reduce(_reduce_properties_logs, properties, {'id': term_id})
+
+    logs = list(map(mapper, blank_nodes))
+
+    return log_as_table(logs)
+
+
 def group_by_keys(group_keys: list = ['term']):
     def run(group: dict, node: dict):
         group_key = '-'.join(non_empty_list(map(lambda v: node.get(v, {}).get('@id'), group_keys)))
@@ -44,7 +106,9 @@ def group_by_keys(group_keys: list = ['term']):
     return run
 
 
-def _module_term_id(term_id: str, module): return getattr(module, 'TERM_ID', term_id).split(',')[0]
+def _module_term_id(term_id: str, module):
+    term_id_str = term_id.split('.')[-1] if '.' in term_id else term_id
+    return getattr(module, 'TERM_ID', term_id_str).split(',')[0]
 
 
 def _run_model_required(model: str, term: dict, data: dict):
@@ -222,12 +286,17 @@ def get_total_value_converted_with_min_ratio(
     total_value_with_property = list_sum([value for term_id, value, prop_value in values if prop_value])
     total_value_ratio = total_value_with_property / total_value if total_value > 0 else 0
 
-    debugValues(node, model=model, term=term, property_id=prop_id,
-                total_value=total_value,
-                total_value_with_property=total_value_with_property,
-                total_value_ratio=total_value_ratio,
-                min_value_ratio=min_ratio,
-                conversion_details=value_logs)
+    logs = {
+        f"{prop_id}-term-id": prop_id,
+        f"{prop_id}-total-value": total_value,
+        f"{prop_id}-total-value-with-property": total_value_with_property,
+        f"{prop_id}-total-value-with-ratio": total_value_ratio,
+        f"{prop_id}-min-value-ratio": min_ratio,
+        f"{prop_id}-values": value_logs
+    }
+
+    debugValues(node, model=model, term=term,
+                **logs)
 
     return list_sum([
         value * prop_value for term_id, value, prop_value in values if all([value, prop_value])
@@ -340,111 +409,6 @@ def convert_to_carbon(node: dict, model: str, term_id: str, blank_nodes: list, *
     ]) if len(missing_carbon_property) == 0 else None
 
 
-class ArrayTreatment(Enum):
-    """
-    Enum representing different treatments for arrays of values.
-    """
-    MEAN = 'mean'
-    MODE = 'mode'
-    SUM = 'sum'
-    FIRST = 'first'
-    LAST = 'last'
-
-
-def _should_run_array_treatment(value):
-    return isinstance(value, Iterable) and len(value) > 0
-
-
-DEFAULT_ARRAY_TREATMENT = ArrayTreatment.MEAN
-ARRAY_TREATMENT_TO_REDUCER = {
-    ArrayTreatment.MEAN: lambda value: mean(value) if _should_run_array_treatment(value) else 0,
-    ArrayTreatment.MODE: lambda value: mode(value) if _should_run_array_treatment(value) else 0,
-    ArrayTreatment.SUM: lambda value: sum(value) if _should_run_array_treatment(value) else 0,
-    ArrayTreatment.FIRST: lambda value: value[0] if _should_run_array_treatment(value) else 0,
-    ArrayTreatment.LAST: lambda value: value[-1] if _should_run_array_treatment(value) else 0
-}
-"""
-A dictionary mapping ArrayTreatment enums to corresponding reducer functions.
-"""
-
-
-def _retrieve_array_treatment(
-    node: dict,
-    is_larger_unit: bool = False,
-    default: ArrayTreatment = ArrayTreatment.FIRST
-) -> ArrayTreatment:
-    """
-    Retrieves the array treatment for a given node.
-
-    Array treatments are used to reduce an array's list of values into
-    a single value. The array treatment is retrieved from a lookup on
-    the node's term.
-
-    Parameters
-    ----------
-    node : dict
-        The dictionary representing the node.
-    is_larger_unit : bool, optional
-        Flag indicating whether to use the larger unit lookup, by default `False`.
-    default : ArrayTreatment, optional
-        Default value to return if the lookup fails, by default `ArrayTreatment.FIRST`.
-
-    Returns
-    -------
-    ArrayTreatment
-        The retrieved array treatment.
-
-    """
-    ARRAY_TREATMENT_LOOKUPS = [
-        'arrayTreatmentLargerUnitOfTime',
-        'arrayTreatment'
-    ]
-    lookup = ARRAY_TREATMENT_LOOKUPS[0] if is_larger_unit else ARRAY_TREATMENT_LOOKUPS[1]
-
-    term = node.get('term', {})
-    lookup_value = get_lookup_value(term, lookup, skip_debug=True)
-
-    return next(
-        (treatment for treatment in ArrayTreatment if treatment.value == lookup_value),
-        default
-    )
-
-
-def get_node_value(
-    node: dict,
-    is_larger_unit: bool = False,
-    array_treatment: Optional[ArrayTreatment] = None,
-    default: Any = 0
-) -> Union[float, bool]:
-    """
-    Get the value from the dictionary representing the node,
-    applying optional array treatment if the value is a list.
-
-    Parameters
-    ----------
-    node : dict
-        The dictionary representing the node.
-    is_larger_unit : bool, optional
-        A flag indicating whether the unit of time is larger, by default `False`.
-    array_treatment : ArrayTreatment, optional
-        An optional override for the treatment to be applied to an array value, if `None` the array treatment in the
-        node's term's lookup is used (which defaults to `FIRST` if no array treatment is specified), by default `None`.
-
-    Returns
-    -------
-    float | bool
-        The extracted value from the node.
-
-    """
-    value = node.get("value", 0)
-
-    reducer = ARRAY_TREATMENT_TO_REDUCER[(
-        array_treatment or _retrieve_array_treatment(node, is_larger_unit=is_larger_unit)
-    )] if isinstance(value, list) and len(value) > 0 else None
-
-    return reducer(value) if reducer else value if isinstance(value, bool) else value or default
-
-
 def _convert_to_set(
     variable: Union[Iterable[Any], Any]
 ) -> set:
@@ -461,8 +425,7 @@ def _convert_to_set(
     set
         A set containing the elements of the input variable.
     """
-    is_iterable = isinstance(variable, Iterable) and not isinstance(variable, (str, bytes))
-    return set(variable) if is_iterable else {variable}
+    return set(variable) if is_iterable(variable) else {variable}
 
 
 def node_term_match(
@@ -559,7 +522,7 @@ def cumulative_nodes_match(
     """
     values = [
         get_node_value(
-            node, is_larger_unit, array_treatment
+            node, 'value', is_larger_unit, array_treatment
         ) or default_node_value for node in nodes if function(node)
     ]
 
@@ -673,7 +636,7 @@ def cumulative_nodes_lookup_match(
 
 class DatestrFormat(Enum):
     """
-    Enum representing ISO date formats permitted by Hestia.
+    Enum representing ISO date formats permitted by HESTIA.
 
     See: https://en.wikipedia.org/wiki/ISO_8601
     """
@@ -738,7 +701,7 @@ def _check_datestr_format(datestr: str, format: DatestrFormat) -> bool:
 
 def _get_datestr_format(datestr: str, default: Optional[Any] = None) -> Union[DatestrFormat, Any, None]:
     """
-    Check a datestr against each ISO format permitted by the Hestia schema and
+    Check a datestr against each ISO format permitted by the HESTIA schema and
     return the matching format.
     """
     return next(
@@ -994,33 +957,6 @@ def _validate_time_fraction_dict(
     ])
 
 
-def _build_update_dict(node: dict, years: list, target_year: int) -> dict:
-    """
-    Build an update dictionary containing values and dates from a node that fall within a given year.
-
-    This is only required if when `group_nodes_by_year` `mode = GroupNodesByYearMode.DATES`
-
-    Parameters
-    ----------
-    node : dict
-        The node containing values and dates.
-    year : int
-        The year to be matched.
-
-    Returns
-    -------
-    dict
-        An update dictionary containing "value" and "dates" keys.
-    """
-    valid_indices = {
-        i for i, y in enumerate(years) if y == target_year
-    }
-    return {
-        "value": [node.get("value")[i] for i in valid_indices],
-        "dates": [node.get("dates")[i] for i in valid_indices]
-    }
-
-
 def group_nodes_by_year(
     nodes: list[dict],
     default_node_duration: int = 1,
@@ -1056,7 +992,7 @@ def group_nodes_by_year(
     should_run_node = GROUP_NODES_BY_YEAR_MODE_TO_SHOULD_RUN_NODE_FUNCTION[mode]
     get_node_datetime_range = GROUP_NODES_BY_YEAR_MODE_TO_GET_DATETIME_RANGE_FUNCTION[mode]
 
-    valid_nodes = [node for node in nodes if should_run_node(node)]
+    valid_nodes = non_empty_list(flatten(split_node_by_dates(node) for node in nodes if should_run_node(node)))
 
     def group_node(groups: dict, index: int):
         node = valid_nodes[index]
@@ -1065,12 +1001,6 @@ def group_nodes_by_year(
             node, default_node_duration=default_node_duration
         )
 
-        # pre-parse the "dates" field so it doesn't get re-calculated in each iteration of the for-loop
-        years = (
-            [safe_parse_date(datestr).year for datestr in node.get("dates", [])]
-            if mode == GroupNodesByYearMode.DATES else []
-        )
-
         range_start = node_datetime_range.start.year if node_datetime_range else 0
         range_end = node_datetime_range.end.year + 1 if node_datetime_range else 0
 
@@ -1084,7 +1014,6 @@ def group_nodes_by_year(
             is_final_year = _datetime_within_range(node_datetime_range.end, group_datetime_range)
 
             time_fraction_dict = _build_time_fraction_dict(group_datetime_range, node_datetime_range)
-            update_dict = _build_update_dict(node, years, year) if mode == GroupNodesByYearMode.DATES else {}
 
             should_run = (
                 mode == GroupNodesByYearMode.DATES
@@ -1095,7 +1024,7 @@ def group_nodes_by_year(
             )
 
             should_run and groups[year].append(
-                node | time_fraction_dict | update_dict
+                node | time_fraction_dict
             )
 
         return groups
@@ -1110,6 +1039,57 @@ def group_nodes_by_year(
     return dict(sorted(iterated.items())) if sort_result else iterated
 
 
+def split_node_by_dates(node: dict) -> list[dict]:
+    """
+    Split a node with an array-like `value` and `dates` with multiple elements into a list of nodes with a single
+    `value` and `dates`. All other array-like node fields (`sd`, `min`, `max`,  and `observations`) will be also be
+    split. Any other fields will be copied with no modifications.
+
+    All split fields will still be array-like, but will only contain one element. Any array-like fields with a
+    different number of elements to `value` will not be split.
+
+    This function should only run on nodes with array-like `value` and `dates` (e.g., nodes with `@type` == `Emission`,
+    `Input`,`Measurement`, `Practice` or `Product`).
+
+    Parameters
+    ----------
+    node : dict
+        A HESTIA blank node with array-like `value` and `dates` (and optional array-like fields `sd`, `min`, `max`, and
+        `observations`).
+
+    Returns
+    -------
+    list[dict]
+        A list of nodes with single `value` and `dates`.
+    """
+    REQUIRED_KEYS = ["value", "dates"]
+    OPTIONAL_KEYS = ["sd", "min", "max", "observations"]
+
+    value = node.get("value", [])
+    target_len = len(value) if isinstance(value, list) else -1
+
+    def should_run_key(key: str) -> bool:
+        item = node.get(key, [])
+        return isinstance(item, list) and len(item) == target_len
+
+    should_run = all([
+        target_len > 0,
+        all(should_run_key(key) for key in REQUIRED_KEYS)
+    ])
+
+    valid_keys = REQUIRED_KEYS + [key for key in OPTIONAL_KEYS if should_run_key(key)]
+
+    def split(result: list[dict], index: int) -> list[dict]:
+        update = {key: [node[key][index]] for key in valid_keys}
+        result.append(node | update)
+        return result
+
+    return (
+        sorted(reduce(split, range(len(value)), list()), key=lambda node: node.get("dates", []))
+        if should_run else [node]
+    )
+
+
 def group_nodes_by_year_and_month(
     nodes: list[dict],
     default_node_duration: int = 1,
@@ -1214,7 +1194,7 @@ def group_nodes_by_last_date(nodes: list) -> dict[str, list[dict]]:
     Parameters
     ----------
     nodes : list[dict]
-        A list of Hestia format nodes.
+        A list of HESTIA format nodes.
 
     Return
     ------
@@ -1229,3 +1209,126 @@ def group_nodes_by_last_date(nodes: list) -> dict[str, list[dict]]:
         return group | {key: group.get(key, []) + [node]}
 
     return reduce(group_by, nodes, {})
+
+
+def get_inputs_from_properties(input: dict, term_types: Union[TermTermType, List[TermTermType]]):
+    """
+    Compute a list of inputs from the input properties, in the `key:value` form.
+
+    Parameters
+    ----------
+    input : dict
+        The Input.
+    term_types : TermTermType | List[TermTermType]
+        List of `termType` valid for the properties `key`.
+
+    Return
+    ------
+    dict
+        A dictionary of nodes grouped by latest date, in the format `{date: list[node]}`.
+    """
+    input_value = list_sum(input.get('value', []))
+    properties = input.get('properties') or download_hestia(input.get('term', {}).get('@id')).get('defaultProperties')
+    inputs = non_empty_list([
+        {
+            'term': p.get('key'),
+            'value': [(p.get('value') / 100) * (p.get('share', 100) / 100) * input_value]
+        } for p in (properties or []) if all([p.get('key'), p.get('value')])
+    ]) if input_value > 0 else []
+    return filter_list_term_type(inputs, term_types)
+
+
+def _get_condensed_nodes(nodes: list) -> tuple[list, bool]:
+    """Only considers nodes which already match on non-date criteria."""
+    CONDENSABLE_UNITS = [Units.BOOLEAN.value, Units.PERCENTAGE_AREA.value]
+    condensed_nodes = []
+    matched_uuids = set()
+    nodes_by_start_date = {_full_date_str(date_str=n["startDate"], is_end=True): n for n in nodes if "startDate" in n}
+    if len(nodes_by_start_date) != len(nodes):
+        return nodes, False
+
+    for node in nodes:
+        search_date = _offset_date(date_str=node.get("endDate", ""))
+        if node["uuid"] in matched_uuids:
+            continue
+        if (search_date in nodes_by_start_date and nodes_by_start_date[search_date]["uuid"] not in matched_uuids
+                and node.get("term", {}).get("units") in CONDENSABLE_UNITS):
+            new_node = node.copy()
+            new_node["endDate"] = nodes_by_start_date[search_date]["endDate"]
+            condensed_nodes.append(new_node)
+            matched_uuids.add(nodes_by_start_date[search_date]["uuid"])
+        elif node["uuid"] not in matched_uuids:
+            condensed_nodes.append(node)
+
+    return condensed_nodes, len(matched_uuids) > 0
+
+
+def condense_nodes(nodes: list) -> list:
+    grouped_nodes = _group_nodes_by_term_and_value(nodes)
+    condensed_nodes = dict()
+    any_changes_made = False
+
+    for key, node_group in grouped_nodes.items():
+        condensed_nodes[key] = node_group
+        while len(condensed_nodes[key]) > 1:
+            condensed_nodes[key], changes_made = _get_condensed_nodes(condensed_nodes[key])
+            if not changes_made:
+                break
+            any_changes_made = True
+
+    if not any_changes_made:
+        return [_omit(values=n, keys=["uuid"]) for n in nodes]
+
+    return sorted(
+        flatten([_omit(values=n, keys=["uuid"]) for nodes in condensed_nodes.values() for n in nodes]),
+        key=lambda x: x["startDate"]
+    )
+
+
+DATE_FORMAT = "%Y-%m-%d"
+
+
+def _variable_length_str_to_date(date_str: str, is_end: bool) -> datetime:
+    """Converts to date, adding start or end of year to YYYY strings as indicated by is_end."""
+    return datetime.strptime(_full_date_str(date_str, is_end=is_end), DATE_FORMAT)
+
+
+def _full_date_str(date_str: str, is_end: bool) -> str:
+    suffix = ""
+    if len(date_str) == 4:
+        # Format YYYY
+        suffix = "-12-31" if is_end else "-01-01"
+    elif len(date_str) == 7:
+        # Format YYYY-MM
+        suffix = f"-{calendar.monthrange(int(date_str[:4]), int(date_str[5:7]))[1]}" if is_end else "-01"
+
+    return date_str + suffix
+
+
+def _with_full_dates(node: dict) -> dict:
+    output_node = node.copy()
+    if "startDate" in output_node:
+        output_node["startDate"] = _full_date_str(output_node["startDate"], is_end=False)
+    if "endDate" in output_node:
+        output_node["endDate"] = _full_date_str(output_node["endDate"], is_end=True)
+
+    return output_node
+
+
+def _offset_date(date_str: str, days: int = 1, is_end: bool = True) -> str:
+    return (
+        _variable_length_str_to_date(date_str=date_str, is_end=is_end) + timedelta(days=days)
+    ).strftime(DATE_FORMAT)
+
+
+def _group_nodes_by_term_and_value(nodes: list) -> dict:
+    grouped_nodes = defaultdict(list)
+
+    for node in nodes:
+        term_id = node.get("term", {}).get("@id", "")
+        value = "-".join([str(v) for v in node.get("value")]) if isinstance(node.get("value"), list) \
+            else node.get("value")
+        node["uuid"] = uuid4()
+        grouped_nodes[(term_id, value)].append(_with_full_dates(node))
+
+    return grouped_nodes
