@@ -1,4 +1,4 @@
-from unittest.mock import patch
+from unittest.mock import patch, call
 import json
 from tests.utils import fixtures_path, fake_new_measurement
 
@@ -32,4 +32,18 @@ def test_run(mock_download, *args):
         site = json.load(f)
 
     run(site)
-    mock_download.assert_called_once()
+    base_params = {'ee_type': 'raster', 'reducer': 'mean'}
+    mock_download.assert_has_calls([
+        call(TERM_ID, site, base_params | {
+            'collection': 'T_CLAY_v2_depth_1', 'depthUpper': 0, 'depthLower': 20
+        }),
+        call(TERM_ID, site, base_params | {
+            'collection': 'T_CLAY_v2_depth_2', 'depthUpper': 20, 'depthLower': 40
+        }),
+        call(TERM_ID, site, base_params | {
+            'collection': 'T_CLAY_v2_depth_3', 'depthUpper': 40, 'depthLower': 60
+        }),
+        call(TERM_ID, site, base_params | {
+            'collection': 'T_CLAY_v2_depth_4', 'depthUpper': 60, 'depthLower': 80
+        })
+    ])
