@@ -19,9 +19,9 @@ def test_region_level_1_id():
 
 
 @patch(f"{class_path}.find_related", return_value=[CYCLE])
-@patch(f"{class_path}.download_hestia", return_value=CYCLE)
+@patch(f"{class_path}._load_calculated_node", return_value=CYCLE)
 def test_related_cycles(*args):
-    assert related_cycles('id') == [CYCLE]
+    assert related_cycles({'@id': 'id'}) == [CYCLE]
 
 
 def test_valid_site_type():
